import logging

from rest_framework import generics, status
from rest_framework.response import Response
from rest_framework.views import APIView

from phone_confirmation.models import PhoneConfirmation
from phone_confirmation.serializers import (ActivationKeySerializer,
                                            ConfirmationSerializer)

logger = logging.getLogger(__name__)


class ConfirmationView(generics.CreateAPIView):
    serializer_class = ConfirmationSerializer
    throttle_scope = 'phone-confirmation-confirmation'


class ActivationKeyView(APIView):
    throttle_scope = 'phone-confirmation-activation-key'

    def post(self, request, *args, **kwargs):
        serializer = ActivationKeySerializer(data=request.data, context={'user': request.user})
        if serializer.is_valid():
            return Response(status=status.HTTP_200_OK, data=serializer.data)
        else:
            return Response(status=status.HTTP_400_BAD_REQUEST, data=serializer.errors)


class GetActivationKeyView(APIView):
    throttle_scope = 'phone-confirmation-activation-key'

    def get(self, request, activation_key, *args, **kwargs):
        try:
            phone_number = PhoneConfirmation.objects.validate_key(activation_key)
            if phone_number:
                return Response(status=status.HTTP_200_OK, data={'phone_number': phone_number})
        except Exception:
            logger.exception('Error decoding activation key')

        return Response(status=status.HTTP_400_BAD_REQUEST, data={'error': 'Invalid activation key'})



from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from phone_confirmation.models import PhoneConfirmation


class ConfirmationSerializer(serializers.ModelSerializer):

    class Meta:
        model = PhoneConfirmation
        fields = ('phone_number',)


class ActivationKeySerializer(serializers.ModelSerializer):

    class Meta:
        model = PhoneConfirmation
        fields = ('phone_number', 'code', 'activation_key')
        extra_kwargs = {
            'phone_number': {'write_only': True, 'required': True},
            'code': {'write_only': True},
            'activation_key': {'read_only': True}
        }

    def is_valid(self, raise_exception=False):
        is_valid = super(ActivationKeySerializer, self).is_valid(raise_exception=raise_exception)
        self.instance = self.Meta.model.objects.get_confirmation_code(phone_number=self.validated_data['phone_number'],
                                                                      code=self.validated_data['code'])
        if is_valid:
            if self.instance:
                user = self.context.get('user')
                self.instance.send_activation_key_created_signal(user=user)
                self.Meta.model.objects.clear_phone_number_confirmations(phone_number=self.validated_data['phone_number'])
            else:
                raise serializers.ValidationError({"error": _("The code or phone number are invalid.")})
        return is_valid



from django.conf.urls import url

from phone_confirmation import views


urlpatterns = [
    url(r'^confirmation/$', views.ConfirmationView.as_view(), name='confirmation'),
    url(r'^activation-key/$', views.ActivationKeyView.as_view(), name='activation-key'),
    url(r'^activation-key/(?P<activation_key>[\w:.-]+)/$', views.GetActivationKeyView.as_view(), name='view-activation-key')
]



from django.contrib import admin

from phone_confirmation.models import PhoneConfirmation


@admin.register(PhoneConfirmation)
class PhoneConfirmation(admin.ModelAdmin):
    list_display = ('created_at', 'phone_number', 'code')
    list_filter = ('created_at',)
    search_fields = ('phone_number',)



from __future__ import unicode_literals

import logging
from datetime import timedelta

from django.conf import settings
from django.core import signing
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils import timezone
from phone_confirmation.fields import RandomPinField
from phone_confirmation.signals import confirmation_sms_sent, activation_key_created
from phonenumber_field.modelfields import PhoneNumberField
from sendsms import api

logger = logging.getLogger(__name__)


phone_settings = getattr(settings, 'PHONE_CONFIRMATION', {})


SALT = phone_settings.get('SALT', 'phonenumber')
ACTIVATION_TIMEOUT = phone_settings.get('ACTIVATION_TIMEOUT', 15 * 60)  # Seconds
SMS_MESSAGE = phone_settings.get('SMS_MESSAGE', 'Your confirmation code is %(code)s')
FROM_NUMBER = phone_settings.get('FROM_NUMBER', '')
MAX_CONFIRMATIONS = phone_settings.get('MAX_CONFIRMATIONS', 10)
SILENT_CONFIRMATIONS_FILTER = phone_settings.get('SILENT_CONFIRMATIONS_FILTER', None)
CODE_LENGTH = phone_settings.get('CONFIRMATION_CODE_LENGTH', 6)


class PhoneConfirmationManager(models.Manager):
    """Manager for PhoneConfirmation model."""

    def validate_key(self, activation_key):
        """
        Check if the activation_key is valid and not expired.

        Return phone number if it is valid or None otherwise.
        """
        try:
            phone_number = signing.loads(
                activation_key,
                salt=SALT,
                max_age=ACTIVATION_TIMEOUT
            )
            return phone_number.get('phone_number')
        except signing.BadSignature:
            return None

    def get_confirmation_code(self, phone_number, code):
        """Get the PhoneConfirmation for the phone number and code."""
        time_threshold = timezone.now() - timedelta(minutes=ACTIVATION_TIMEOUT)
        return self.get_queryset().filter(created_at__gte=time_threshold,
                                          phone_number=phone_number,
                                          code=code).order_by('-created_at').first()

    def clear_phone_number_confirmations(self, phone_number):
        """Remove all confirmations for the phone number."""
        self.get_queryset().filter(phone_number=phone_number).delete()


class PhoneConfirmation(models.Model):
    """Store confirmation codes for phone number."""
    created_at = models.DateTimeField(auto_now_add=True)
    phone_number = PhoneNumberField(db_index=True)
    code = RandomPinField(length=CODE_LENGTH)
    objects = PhoneConfirmationManager()

    class Meta:
        index_together = (('created_at', 'phone_number', 'code'), ('phone_number', 'code'))

    @property
    def activation_key(self):
        return self._get_activation_key(self.phone_number)

    def __str__(self):
        return str(self.phone_number)

    @staticmethod
    def _get_activation_key(phone_number):
        return signing.dumps(
            obj={'phone_number': str(phone_number)},
            salt=SALT
        )

    def _send_signal_and_log(self, signal, **kwargs):
        """Replacement for Signal.send_robust with logging"""
        rvs = signal.send_robust(sender=self.__class__, **kwargs)
        for f, exc in rvs:
            if exc is not None:
                logger.error("signal handler %s failed on %r",
                             getattr(f, '__name__', f), self,
                             exc_info=(type(exc), exc, exc.__traceback__))

    def send_sms(self, request=None):
        message = SMS_MESSAGE % {'code': self.code}
        to = str(self.phone_number)
        if callable(SILENT_CONFIRMATIONS_FILTER) and SILENT_CONFIRMATIONS_FILTER(to) is True:
            logger.debug("Filtered phone confirmation SMS to:%s text:%s", to, message)
            return

        api.send_sms(body=message,
                     from_phone=FROM_NUMBER,
                     to=[to])
        self._send_signal_and_log(confirmation_sms_sent, phone_number=self.phone_number)

    def send_activation_key_created_signal(self, user=None):
        self._send_signal_and_log(activation_key_created, phone_number=self.phone_number, activation_key=self.activation_key, user=user)


@receiver(post_save, sender=PhoneConfirmation)
def post_save_phone_confirmation_receiver(sender, instance, created, **kwargs):
    if created:
        if PhoneConfirmation.objects.filter(phone_number=instance.phone_number).count() > MAX_CONFIRMATIONS:
            PhoneConfirmation.objects.filter(phone_number=instance.phone_number).order_by('created_at').first().delete()
        instance.send_sms()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-14 03:19
from __future__ import unicode_literals

import random

from django.db import migrations, models
import phonenumber_field.modelfields


def random_confirmation_code():
    return '{0:04d}'.format(random.SystemRandom().randint(1, 9999))


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PhoneConfirmation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('phone_number', phonenumber_field.modelfields.PhoneNumberField(db_index=True, max_length=128)),
                ('code', models.CharField(default=random_confirmation_code, max_length=6)),
            ],
        ),
        migrations.AlterIndexTogether(
            name='phoneconfirmation',
            index_together=set([('phone_number', 'code'), ('created_at', 'phone_number', 'code')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-07-14 21:03
from __future__ import unicode_literals

from django.db import migrations
import phone_confirmation.fields


class Migration(migrations.Migration):

    dependencies = [
        ('phone_confirmation', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='phoneconfirmation',
            name='code',
            field=phone_confirmation.fields.RandomPinField(blank=True, length=6, max_length=6),
        ),
    ]



# Based on https://github.com/bjldigital/django-randompinfield/blob/master/randompinfield/fields.py
from random import randint

from django.db import models


class RandomPinField(models.CharField):
    """
    Generates a random digit pin
    By default sets length=6
    """
    def __init__(self, length, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['max_length'] = length
        self.length = length
        super(RandomPinField, self).__init__(*args, **kwargs)

    def generate_pin(self):
        """
        Returns a random pin.
        """
        range_start = 10**(self.max_length - 1)
        range_end = (10**self.max_length) - 1

        return str(randint(range_start, range_end))

    def pre_save(self, instance, add):
        value = getattr(instance, self.attname)
        if not value:
            value = self.generate_pin()
            setattr(instance, self.attname, value)
        return value

    def deconstruct(self):
        name, path, args, kwargs = super(RandomPinField, self).deconstruct()
        kwargs['length'] = self.length
        return name, path, args, kwargs
